@@ -1,3 +1,4 @@
+from collections import defaultdict
 from typing import Dict, List, Optional, Union
 
 from ..data_model import *
@@ -21,7 +22,8 @@ class WorkspaceClient(ServiceClient):
         :raises WorkspaceNotSet: If neither ``workspace`` nor
             :data:`Beaker.config.default_workspace <beaker.Config.default_workspace>` are set.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         workspace = workspace or self.config.default_workspace
         if workspace is None:
@@ -61,7 +63,8 @@ class WorkspaceClient(ServiceClient):
         :raises ValueError: If the workspace name is invalid.
         :raises WorkspaceConflict: If a workspace by that name already exists.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         workspace_name = self.resolve_workspace_name(workspace)
         org, name = workspace_name.split("/", 1)
@@ -84,7 +87,8 @@ class WorkspaceClient(ServiceClient):
 
         :raises ValueError: If the workspace name is invalid.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         try:
             return self.get(workspace)
@@ -99,7 +103,8 @@ class WorkspaceClient(ServiceClient):
 
         :raises WorkspaceNotFound: If the workspace doesn't exist.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         if workspace is None:  # could accidentally archive default workspace if None
             raise TypeError("Expected 'str', got 'NoneType'")
@@ -124,7 +129,8 @@ class WorkspaceClient(ServiceClient):
 
         :raises WorkspaceNotFound: If the workspace doesn't exist.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         if workspace is None:  # could accidentally unarchive default workspace if None
             raise TypeError("Expected 'str', got 'NoneType'")
@@ -151,7 +157,8 @@ class WorkspaceClient(ServiceClient):
         :raises WorkspaceNotFound: If the workspace doesn't exist.
         :raises ValueError: If the new name is invalid.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         self.validate_beaker_name(name)
         if workspace is None:  # could accidentally rename default workspace if None
@@ -186,7 +193,8 @@ class WorkspaceClient(ServiceClient):
         :raises WorkspaceNotSet: If neither ``workspace`` nor
             :data:`Beaker.config.default_workspace <beaker.Config.default_workspace>` are set.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         workspace_name = self.resolve_workspace(workspace).full_name
         self.request(
@@ -224,7 +232,8 @@ class WorkspaceClient(ServiceClient):
             :data:`Beaker.config.default_org <beaker.Config.default_org>` are set.
         :raises AccountNotFound: If the author account doesn't exist.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         org = self.resolve_org(org)
         workspaces: List[Workspace] = []
@@ -278,7 +287,8 @@ class WorkspaceClient(ServiceClient):
         :raises WorkspaceNotSet: If neither ``workspace`` nor
             :data:`Beaker.config.default_workspace <beaker.Config.default_workspace>` are set.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         workspace_name = self.resolve_workspace(workspace, read_only_ok=True).full_name
         images: List[Image] = []
@@ -327,7 +337,8 @@ class WorkspaceClient(ServiceClient):
         :raises WorkspaceNotSet: If neither ``workspace`` nor
             :data:`Beaker.config.default_workspace <beaker.Config.default_workspace>` are set.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         workspace_name = self.resolve_workspace(workspace, read_only_ok=True).full_name
         experiments: List[Experiment] = []
@@ -380,7 +391,8 @@ class WorkspaceClient(ServiceClient):
         :raises WorkspaceNotSet: If neither ``workspace`` nor
             :data:`Beaker.config.default_workspace <beaker.Config.default_workspace>` are set.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         workspace_name = self.resolve_workspace(workspace, read_only_ok=True).full_name
         datasets: List[Dataset] = []
@@ -426,7 +438,8 @@ class WorkspaceClient(ServiceClient):
         :raises WorkspaceNotSet: If neither ``workspace`` nor
             :data:`Beaker.config.default_workspace <beaker.Config.default_workspace>` are set.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         workspace_name = self.resolve_workspace(workspace, read_only_ok=True).full_name
         return [
@@ -451,7 +464,8 @@ class WorkspaceClient(ServiceClient):
         :raises WorkspaceNotSet: If neither ``workspace`` nor
             :data:`Beaker.config.default_workspace <beaker.Config.default_workspace>` are set.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         workspace_name = self.resolve_workspace(workspace, read_only_ok=True).full_name
         return [
@@ -478,7 +492,8 @@ class WorkspaceClient(ServiceClient):
         :raises WorkspaceNotSet: If neither ``workspace`` nor
             :data:`Beaker.config.default_workspace <beaker.Config.default_workspace>` are set.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         workspace_name = self.resolve_workspace(workspace, read_only_ok=True).full_name
         return WorkspacePermissions.from_json(
@@ -511,7 +526,8 @@ class WorkspaceClient(ServiceClient):
         :raises WorkspaceNotSet: If neither ``workspace`` nor
             :data:`Beaker.config.default_workspace <beaker.Config.default_workspace>` are set.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         if auth not in set(Permission):
             raise ValueError(f"Authorization '{auth}' is invalid")
@@ -543,7 +559,8 @@ class WorkspaceClient(ServiceClient):
         :raises WorkspaceNotSet: If neither ``workspace`` nor
             :data:`Beaker.config.default_workspace <beaker.Config.default_workspace>` are set.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         workspace_name = self.resolve_workspace(workspace, read_only_ok=True).full_name
         self.request(
@@ -569,7 +586,8 @@ class WorkspaceClient(ServiceClient):
         :raises WorkspaceNotSet: If neither ``workspace`` nor
             :data:`Beaker.config.default_workspace <beaker.Config.default_workspace>` are set.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         account_ids = [
             account.id if isinstance(account, Account) else self.beaker.account.get(account).id
@@ -600,6 +618,78 @@ class WorkspaceClient(ServiceClient):
         workspace_name = self.resolve_workspace(workspace, read_only_ok=True).full_name
         return f"{self.config.agent_address}/ws/{workspace_name}"
 
+    def clear(
+        self,
+        workspace: Optional[Union[str, Workspace]] = None,
+        groups: bool = True,
+        experiments: bool = True,
+        images: bool = True,
+        datasets: bool = True,
+        secrets: bool = True,
+    ):
+        """
+        Remove groups, experiments, images, datasets, and secrets from a workspace.
+
+        :param workspace: The Beaker workspace name, or object. If not specified,
+            :data:`Beaker.config.default_workspace <beaker.Config.default_workspace>` is used.
+        :param groups: Whether to delete groups.
+        :param experiments: Whether to delete experiments.
+        :param images: Whether to delete images.
+        :param datasets: Whether to delete datasets.
+        :param secrets: Whether to delete secrets.
+
+        :raises WorkspaceNotFound: If the workspace doesn't exist.
+        :raises WorkspaceNotSet: If neither ``workspace`` nor
+            :data:`Beaker.config.default_workspace <beaker.Config.default_workspace>` are set.
+        :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
+        """
+        import concurrent.futures
+
+        deletion_counts: Dict[str, int] = defaultdict(int)
+        with concurrent.futures.ThreadPoolExecutor() as executor:
+            deletion_futures = []
+
+            if groups:
+                for group in self.groups(workspace):
+                    future = executor.submit(self.beaker.group.delete, group)
+                    deletion_futures.append(future)
+                    deletion_counts["groups_deleted"] += 1
+
+            if experiments:
+                for experiment in self.experiments(workspace):
+                    future = executor.submit(self.beaker.experiment.delete, experiment)
+                    deletion_futures.append(future)
+                    deletion_counts["experiments_deleted"] += 1
+
+            if images:
+                for image in self.images(workspace):
+                    future = executor.submit(self.beaker.image.delete, image)
+                    deletion_futures.append(future)
+                    deletion_counts["images_deleted"] += 1
+
+            if datasets:
+                for dataset in self.datasets(workspace):
+                    future = executor.submit(self.beaker.dataset.delete, dataset)
+                    deletion_futures.append(future)
+                    deletion_counts["datasets_deleted"] += 1
+
+            if secrets:
+                for secret in self.secrets(workspace):
+                    future = executor.submit(self.beaker.secret.delete, secret, workspace)
+                    deletion_futures.append(future)
+                    deletion_counts["secrets_deleted"] += 1
+
+            done, _ = concurrent.futures.wait(deletion_futures)
+            for future in done:
+                try:
+                    future.result()
+                except NotFoundError:
+                    pass
+
+        return WorkspaceClearResult(**deletion_counts)
+
     def _not_found_err_msg(self, workspace: str) -> str:
         return (
             f"'{workspace}': Make sure you're using the workspace ID or *full* name "
