@@ -2,13 +2,12 @@ import re
 import time
 from collections import OrderedDict
 from datetime import datetime, timedelta
+from functools import wraps
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, Callable, Optional, Tuple, TypeVar, Union
+from typing import Any, Callable, Optional, Tuple, Type, TypeVar, Union
 
 from .aliases import PathOrStr
-
-if TYPE_CHECKING:
-    from .services.service_client import ServiceClient
+from .exceptions import RequestException
 
 
 def to_lower_camel(s: str) -> str:
@@ -60,9 +59,9 @@ def cached_property(ttl: float = 60):
     See :meth:`~beaker.services.account.AccountClient.name`, for example.
     """
 
-    def ttl_cached_property(prop: Callable[[Any], T]):
+    def ttl_cached_property(prop) -> property:
         @property  # type: ignore[misc]
-        def prop_with_cache(self: "ServiceClient") -> T:
+        def prop_with_cache(self):
             key = (prop.__qualname__, repr(self.config))
             cached = _property_cache.get(key)
             if cached is not None:
@@ -75,7 +74,7 @@ def cached_property(ttl: float = 60):
                 _property_cache.popitem(last=False)
             return value
 
-        return prop_with_cache
+        return prop_with_cache  # type: ignore[return-value]
 
     return ttl_cached_property
 
@@ -101,3 +100,42 @@ def split_timestamp(s: bytes) -> Optional[str]:
         return match.group(1).decode()
     else:
         return None
+
+
+def log_and_wait(retries_so_far: int, err: Exception) -> None:
+    from .client import Beaker
+
+    retry_in = min(Beaker.BACKOFF_FACTOR * (2**retries_so_far), Beaker.BACKOFF_MAX)
+    Beaker.logger.debug("Request failed with: %s\nRetrying in %d seconds...", err, retry_in)
+    time.sleep(retry_in)
+
+
+def retriable(
+    on_failure: Optional[Callable[..., None]] = None,
+    recoverable_errors: Tuple[Type[Exception], ...] = (RequestException,),
+):
+    """
+    Use to make a service client method more robust by allowing retries.
+    """
+
+    def parametrize_decorator(func: Callable[..., T]) -> Callable[..., T]:
+        @wraps(func)
+        def retriable_method(*args, **kwargs) -> T:
+            from .client import Beaker
+
+            retries = 0
+            while True:
+                try:
+                    return func(*args, **kwargs)
+                except recoverable_errors as err:
+                    if retries < Beaker.MAX_RETRIES:
+                        if on_failure is not None:
+                            on_failure()
+                        log_and_wait(retries, err)
+                        retries += 1
+                    else:
+                        raise
+
+        return retriable_method
+
+    return parametrize_decorator
